/*
 * vga.c: VGA code
 */

#include <dpmi.h>
#include <go32.h>
#include <pc.h>
#include <stdio.h>
#include <sys/farptr.h>
#include "genital.h"
#include "misc.h"
#include "vdp.h"
#include "vga.h"

int                     video_mode = TEXTMODE;


void SetPalette(unsigned char color, unsigned short int rgb)
{
        outportb(PEL_WRITE_ADDR_REG, color >> 1);       /* request color */

        /* fix: optimize the shifting here */
        outportb(PEL_DATA_REG, ((rgb >> 1) & 7) << 3);  /* red */
        outportb(PEL_DATA_REG, ((rgb >> 5) & 7) << 3);  /* green */
        outportb(PEL_DATA_REG, ((rgb >> 9) & 7) << 3);  /* blue */
}

void ResetPalette()     /* reload entire palette */
{
        int     i;

        for (i = 0; i < 128; i += 2)
                SetPalette(i, GetWord(&vdp.cram[i]));
}
        
int GraphicsGetMode()
{
        return video_mode;
}

void GraphicsSetMode(int res)
{
        unsigned char   v;
        __dpmi_regs     r;

        r.x.ax = 3;
        __dpmi_int(0x10, &r);

        video_mode = res;

        /* initialize VGA so that it lets us use any register combination */
        outportb(0x3d4, 0x11);
        v = inportb(0x3d5) & 0x7f;
        outportb(0x3d4, 0x11);
        outportb(0x3d5, v);

        outportb(0x3c6, 0xff);  /* make all palette registers accesible */

        if (res == MODE13)      /* mode 0x13: 320x200 linear */
        {                
                r.x.ax = 0x13;
                __dpmi_int(0x10, &r);

                inportb(0x3da);         /* must be done before 0x3c0 access */
                ResetPalette();

                /* set border color to 255 (which will be 0) */
                outportb(ATTRIB_REG, 0x31);     /* index 0x11: border color */
                outportb(ATTRIB_REG, 255);      /* ...bit 5 for screen on */
        }
        else if (res == MODEX)  /* ModeX: 320x240 planar */ 
        {
                /* first enter mode 0x13 */
                r.x.ax = 0x13;
                __dpmi_int(0x10, &r);
                inportb(0x3da);         /* must be done before 0x3c0 access */
                /* next we must "ModeX it" */
                outportw(SEQU_ADDR, 0x0604);
                outportw(CRTC_ADDR, 0xe317);
                outportw(CRTC_ADDR, 0x0014);
                outportw(SEQU_ADDR, 0x0f02);    /* clear VRAM */
                FastMemset((void *) 0xa0000, 255, 65536); /* 255 is always
                                                             black */
                /* now turn it into 320x240 */
                outportb(0x3c2, 0xe3);
                outportw(0x3d4, 0x2c11);
                outportw(0x3d4, 0x0d06);
                outportw(0x3d4, 0x3e07);
                outportw(0x3d4, 0xea10);
                outportw(0x3d4, 0xac11);
                outportw(0x3d4, 0xdf12);
                outportw(0x3d4, 0xe715);
                outportw(0x3d4, 0x0616);

                /* set border color to 255 (which will be 0) */
                outportb(ATTRIB_REG, 0x31);     
                outportb(ATTRIB_REG, 255);      

                ResetPalette();
        }
        else    /* text mode */
        {
                r.x.ax = 3;
                __dpmi_int(0x10, &r);
        }
        outportb(PEL_WRITE_ADDR_REG, 254);      /* color 254 is green */
        outportb(PEL_DATA_REG, 0x00);
        outportb(PEL_DATA_REG, 0xff);
        outportb(PEL_DATA_REG, 0x00);
}

void GraphicsInit(int res)
{
        int             i;        

        if (res == MODE13)
                GraphicsSetMode(MODE13);
        else if (res == MODEX)
                GraphicsSetMode(MODEX);

        /* set all colors to 0 initially */
        for (i = 0; i < 64; i++)
        {
                outportb(PEL_WRITE_ADDR_REG, i); 
                outportb(PEL_DATA_REG, 0);
                outportb(PEL_DATA_REG, 0);
                outportb(PEL_DATA_REG, 0);
        }
}
        
void GraphicsDeinit()
{
        __dpmi_regs     r;

        r.x.ax = 3;
        __dpmi_int(0x10, &r);
}

